var React = require('react');
var Buttons = require('./buttons.js');
var classNames = require("classnames");

var _inFrame = function() {
    try {
        return window.self !== window.top;
    } catch (e) {
        return true;
    }
};

var _getPercentage = function(current, max) {
    if(current === 0 || max === 0 || isNaN(current) || isNaN(max)) {
        return 0;
    } else {
        return ((current / max) * 100).toFixed(2);
    };
};

// Debugging
var _log = function(text, error) {
    if(window.console) {
        console[(error ? "error" : "log")](text);
    }
};

    // Credits: http://paypal.github.io/accessible-html5-video-player/
    // Unfortunately, due to mixed support, UA sniffing is required
    var _browserSniff = function() {
        var nAgt = navigator.userAgent,
            name = navigator.appName,
            fullVersion = "" + parseFloat(navigator.appVersion),
            majorVersion = parseInt(navigator.appVersion, 10),
            nameOffset,
            verOffset,
            ix;

        // MSIE 11
        if ((navigator.appVersion.indexOf("Windows NT") !== -1) && (navigator.appVersion.indexOf("rv:11") !== -1)) {
            name = "IE";
            fullVersion = "11;";
        }
        // MSIE
        else if ((verOffset=nAgt.indexOf("MSIE")) !== -1) {
            name = "IE";
            fullVersion = nAgt.substring(verOffset + 5);
        }
        // Chrome
        else if ((verOffset=nAgt.indexOf("Chrome")) !== -1) {
            name = "Chrome";
            fullVersion = nAgt.substring(verOffset + 7);
        }
        // Safari
        else if ((verOffset=nAgt.indexOf("Safari")) !== -1) {
            name = "Safari";
            fullVersion = nAgt.substring(verOffset + 7);
            if ((verOffset=nAgt.indexOf("Version")) !== -1) {
                fullVersion = nAgt.substring(verOffset + 8);
            }
        }
        // Firefox
        else if ((verOffset=nAgt.indexOf("Firefox")) !== -1) {
            name = "Firefox";
            fullVersion = nAgt.substring(verOffset + 8);
        }
        // In most other browsers, "name/version" is at the end of userAgent 
        else if ((nameOffset=nAgt.lastIndexOf(" ") + 1) < (verOffset=nAgt.lastIndexOf("/"))) {
            name = nAgt.substring(nameOffset,verOffset);
            fullVersion = nAgt.substring(verOffset + 1);

            if (name.toLowerCase() == name.toUpperCase()) {
                name = navigator.appName;
            }
        }
        // Trim the fullVersion string at semicolon/space if present
        if ((ix = fullVersion.indexOf(";")) !== -1) {
            fullVersion = fullVersion.substring(0, ix);
        }
        if ((ix = fullVersion.indexOf(" ")) !== -1) {
            fullVersion = fullVersion.substring(0, ix);
        }
        // Get major version
        majorVersion = parseInt("" + fullVersion, 10);
        if (isNaN(majorVersion)) {
            fullVersion = "" + parseFloat(navigator.appVersion); 
            majorVersion = parseInt(navigator.appVersion, 10);
        }

        // Return data
        return {
            name:       name, 
            version:    majorVersion, 
            ios:        /(iPad|iPhone|iPod)/g.test(navigator.platform)
        };
    };

    // Fullscreen API
    var _fullscreen = function() {
        var fullscreen = {
                supportsFullScreen: false,
                isFullScreen: function() { return false; },
                requestFullScreen: function() {},
                cancelFullScreen: function() {},
                fullScreenEventName: "",
                element: null,
                prefix: ""
            },
            browserPrefixes = "webkit moz o ms khtml".split(" ");

        // check for native support
        if (typeof document.cancelFullScreen != "undefined") {
            fullscreen.supportsFullScreen = true;
        }
        else {
            // check for fullscreen support by vendor prefix
            for (var i = 0, il = browserPrefixes.length; i < il; i++ ) {
                fullscreen.prefix = browserPrefixes[i];

                if (typeof document[fullscreen.prefix + "CancelFullScreen"] != "undefined") {
                    fullscreen.supportsFullScreen = true;
                    break;
                }
                // Special case for MS (when isn't it?)
                else if (typeof document.msExitFullscreen != "undefined" && document.msFullscreenEnabled) {
                    fullscreen.prefix = "ms";
                    fullscreen.supportsFullScreen = true;
                    break;
                }
            }
        }

        // Safari doesn't support the ALLOW_KEYBOARD_INPUT flag (for security) so set it to not supported
        // https://bugs.webkit.org/show_bug.cgi?id=121496
        if(fullscreen.prefix === "webkit" && !!navigator.userAgent.match(/Version\/[\d\.]+.*Safari/)) {
             fullscreen.supportsFullScreen = false;
        }

        // Update methods to do something useful
        if (fullscreen.supportsFullScreen) {
            // Yet again Microsoft awesomeness, 
            // Sometimes the prefix is "ms", sometimes "MS" to keep you on your toes
            fullscreen.fullScreenEventName = (fullscreen.prefix == "ms" ? "MSFullscreenChange" : fullscreen.prefix + "fullscreenchange");

            fullscreen.isFullScreen = function(element) {
                if(typeof element == "undefined") {
                    element = document;
                }

                switch (this.prefix) {
                    case "":
                        return document.fullscreenElement == element;
                    case "moz":
                        return document.mozFullScreenElement == element;
                    default:
                        return document[this.prefix + "FullscreenElement"] == element;
                }
            };
            fullscreen.requestFullScreen = function(element) {
                return (this.prefix === "") ? element.requestFullScreen() : element[this.prefix + (this.prefix == "ms" ? "RequestFullscreen" : "RequestFullScreen")](this.prefix === "webkit" ? element.ALLOW_KEYBOARD_INPUT : null);
            };
            fullscreen.cancelFullScreen = function() {
                return (this.prefix === "") ? document.cancelFullScreen() : document[this.prefix + (this.prefix == "ms" ? "ExitFullscreen" : "CancelFullScreen")]();
            };
            fullscreen.element = function() { 
                return (this.prefix === "") ? document.fullscreenElement : document[this.prefix + "FullscreenElement"];
            };
        }

        return fullscreen;
    };

var Player = React.createClass({

    getInitialState: function() {
        return {
            playing: false,
            percentBuffered: 0,
            duration: 0,
            currentTime: 0,
            muted: false,
            volumeLevel: 0.5,
            fullScreen: false,
            showControls: true,
            loading: false
        };
    },

    /*
     * Playback
     */

    play: function() {
        this.setState({
            playing: true
        });
        this.refs.video.getDOMNode().play();
    },

    pause: function() {
        this.setState({
            playing: false
        });
        this.refs.video.getDOMNode().pause();
    },

    togglePlayback: function() {
        if(this.state.playing) {
            this.pause();
        } else {
            this.play();
        };
    },

    rewind: function(seekTime) {
        if(typeof seekTime !== "number") {
            seekTime = 10;
        };
        this.seek(this.refs.video.getDOMNode().currentTime - seekTime);
    },

    forward: function(seekTime) {
        if(typeof seekTime !== "number") {
            seekTime = 10;
        };
        this.seek(this.refs.video.getDOMNode().currentTime + seekTime);
    },


    //Seek to time
    //The input parameter can be an event or a number
    seek: function(input) {
        var targetTime = 0;

        if(typeof input === "number") {
            targetTime = input;
        } else if(typeof input === "object" && (input.type === "input" || input.type === "change")) {
            //seek slider input
            targetTime = ((input.target.value / input.target.max) * this.refs.video.getDOMNode().duration);
        };

        if(targetTime < 0) {
            targetTime = 0;
        } else if(targetTime > this.refs.video.getDOMNode().duration) {
            targetTime = this.refs.video.getDOMNode().duration;
        };

        try {
            this.refs.video.getDOMNode().currentTime = targetTime.toFixed(1);
        } catch (e) {};
    },

    handleVideoClick: function() {
        if(!this.state.playing) {
            this.play();
        } else if(this.state.duration == this.state.currentTime) {
            this.seek();
            this.play();
        } else {
            this.pause();
        };
    },
    
    checkPlaying: function(event) {
        if(event.type === "play") {
            this.setState({
                playing: true
            });
        } else if(event.type === "pause") {
            this.setState({
                playing: false
            });
        } else if(event.type === "ended") {
            this.setState({
                currentTime: this.state.duration,
                playing: false
            });
        };
    },

    /*
     * Progress
     */

    bufferUpdate: function() {
        var buffered = this.refs.video.getDOMNode().buffered;
        if(buffered.length) {
            this.setState({
                percentBuffered: _getPercentage(buffered.end(0), this.refs.video.getDOMNode().duration),
                duration: this.refs.video.getDOMNode().duration
            });
        };
    },

    timeUpdate: function() {
        this.setState({
            currentTime: this.refs.video.getDOMNode().currentTime,
            duration: this.refs.video.getDOMNode().duration
        });
    },

    generateTimeDisplay: function(time) {
        var secs = parseInt(time % 60);
        var mins = parseInt((time / 60) % 60);
        var hours = parseInt(((time / 60) / 60) % 60);
        var displayHours = (parseInt(((this.duration / 60) / 60) / 60) > 0);
        secs = ("0" + secs).slice(-2);
        mins = ("0" + mins).slice(-2);
        var result = (displayHours ? hours + ":" : "") + mins + ":" + secs;
        return result;
    },
    /*
     * Volume
     */

    toggleMute: function() {
        this.setState({
            muted: !this.state.muted
        }, function() {
            this.refs.video.getDOMNode().muted = this.state.muted;
        });
    },

    setVolume: function(volume) {
        if(typeof volume === "undefined") {
            volume = this.state.volumeLevel;
        };
        volume = parseFloat(volume);
        if(volume > 1) {
            volume = 1;
        };
        if(volume < 0) {
            volume = 0;
        };
        this.setState({
            volumeLevel: volume
        }, function() {
            this.refs.video.getDOMNode().volume = this.state.volumeLevel;
        });
        if(volume > 0 && this.state.muted) {
            this.toggleMute();
        };
        if(volume === 0 && !this.state.muted) {
            this.toggleMute();
        };
    },


    /*
     * Display
     */

    toggleFullScreen: function(event) {
        var nativeSupport = this.fullscreen.supportsFullScreen;
        if(event && event.type === this.fullscreen.fullScreenEventName) {
            this.setState({
                fullScreen: this.fullscreen.isFullScreen(this.getDOMNode())
            });
        } else if(nativeSupport) {
            if(!this.fullscreen.isFullScreen(this.getDOMNode())) {
                this.fullscreen.requestFullScreen(this.getDOMNode());
            } else {
                this.fullscreen.cancelFullScreen();
            };
            this.setState({
                fullScreen: this.fullscreen.isFullScreen(this.getDOMNode())
            });
        } else {
            this.setState({
                fullScreen: !this.state.fullScreen
            }, function() {
                if(this.state.fullScreen) {
                    document.addEventListener("keyup", this.handleEscapeFullscreen.bind(this));
                    document.body.style.overflow = "hidden";
                } else {
                    document.removeEventListener("keyup", this.handleEscapeFullscreen.bind(this));
                    document.body.style.overflow = "";
                };
            });
        };
    },

    handleEscapeFullscreen: function(event) {
        if((event.which || event.charCode || event.keyCode) === 27 && this.state.fullScreen) {
            this.toggleFullScreen();
        };
    },


    toggleControls: function(event) {
        if(event.type === "mouseenter") {
            this.setState({
                showControls: true
            });
        } else {
            this.setState({
                showControls: false
            });
        };
    },

    checkLoading: function(event) {
        var loading = (event.type === "waiting");
        var component = this;
        clearTimeout(this.loadingTimer);
        this.loadingTimer = setTimeout(function() {
            component.setState({
                loading: loading
            });
        }, (loading ? 250 : 0));
    },

    /*
     * Component Lifecycle
     */

    componentWillMount: function() {
        this.fullscreen = _fullscreen();
        this.browser = _browserSniff();
        _log(this.browser.name + " " + this.browser.version);
    },

    componentDidMount: function() {
        document.addEventListener(this.fullscreen.fullScreenEventName, this.toggleFullScreen);
        this.refs.video.getDOMNode().addEventListener("timeupdate", this.timeUpdate);
        this.refs.video.getDOMNode().addEventListener("seeking", this.timeUpdate);
        this.refs.video.getDOMNode().addEventListener("loadedmetadata", this.timeUpdate);
        this.refs.video.getDOMNode().addEventListener("ended", this.checkPlaying);
        this.refs.video.getDOMNode().addEventListener("progress", this.bufferUpdate);
        this.refs.video.getDOMNode().addEventListener("playing", this.bufferUpdate);
        this.refs.video.getDOMNode().addEventListener("play", this.checkPlaying);
        this.refs.video.getDOMNode().addEventListener("pause", this.checkPlaying);
        this.refs.video.getDOMNode().addEventListener("waiting", this.checkLoading);
        this.refs.video.getDOMNode().addEventListener("canplay", this.checkLoading);
        this.refs.video.getDOMNode().addEventListener("seeked", this.checkLoading);
        this.refs.controls.getDOMNode().addEventListener("mouseenter", this.toggleControls);
        this.refs.controls.getDOMNode().addEventListener("mouseleave", this.toggleControls);
        this.refs.video.getDOMNode().load();
        if(this.props.autoplay) {
            this._play();
        };
    },

    render: function() {
        var style = {
            "fontFamily": 'Avenir,"Helvetica Neue",Helvetica,Arial,sans-serif'
        };
        var classnames = classNames({
            'player': true,
            'player-video': true,
            'stopped': !this.state.playing,
            'playing': this.state.playing,
            'ios': this.browser.ios,
            'fullscreen-enabled': this.fullscreen.supportsFullScreen || (this.fullscreen.fallback && ! _inFrame()),
            'fullscreen-hide-controls': true,
            'fullscreen-active': this.state.fullScreen,
            'muted': this.state.muted,
            'loading': this.state.loading
        });
        var controlClassnames = classNames({
            'hover': this.state.showControls,
            'player-controls': true
        });
        var duration = this.generateTimeDisplay(this.state.duration);
        var currentTime = this.generateTimeDisplay(this.state.currentTime);
        var percentPlayed = _getPercentage(this.state.currentTime, this.state.duration);
        var volumeLevel = this.state.muted ? 0 : this.state.volumeLevel;
        return (
            <div style={style} className={classnames}>
                <div onClick={this.handleVideoClick} className="playervideo-wrapper">
                    <Video  ref="video" poster={this.props.poster} video={this.props.video} />
                </div>
                <div ref="controls" className={controlClassnames}>
                <ProgressBar handleSeek={this.seek} percentPlayed={percentPlayed} percentBuffered={this.state.percentBuffered} />
                <span className="player-controls-left">
                    <Buttons.RestartButton handleSeek={this.seek} />
                    <Buttons.RewindButton handleRewind={this.rewind} />
                    <Buttons.PlayButton handleTogglePlayback={this.togglePlayback} playing={this.state.playing} />
                    <Buttons.FastForwardButton handleForward={this.forward} />
                    <span className="player-time">
                        <span className="player-current-time">{currentTime}</span>
                    </span>
                    <span className="player-time">
                        <span className="player-duration">{duration}</span>
                    </span>
                </span>
                <span className="player-controls-right">
                    <Buttons.MuteButton muted={this.state.muted} toggleMute={this.toggleMute} />
                    <Buttons.VolumeControl volumeLevel={volumeLevel} setVolume={this.setVolume} />
                    <Buttons.FullScreenButton onToggleFullScreen={this.toggleFullScreen} fullScreen={this.state.fullScreen} />
                </span>
                </div>
            </div>
        );
    },


});

var Video = React.createClass({

    render: function() {
        if(this.props.poster) {
            return (
                <video poster={this.props.poster} crossOrigin>
                    <source src={this.props.video} />
                </video>
            );
        } else {
            return (
                <video crossOrigin>
                    <source src={this.props.video} />
                </video>
            );
        };
    }
});

var ProgressBar = React.createClass({
    
    seek: function(event) {
        this.props.handleSeek(event);
    },

    render: function() {
        return (
            <div className="player-progress">
                <input id="seek" className="player-progress-seek" type="range" min="0" max="100" step="0.5" value="0" data-player="seek" onChange={this.seek} />
                <progress className="player-progress-played" max="100" value={this.props.percentPlayed}>
                </progress>
                <progress className="player-progress-buffer" max="100" value={this.props.percentBuffered}>
                </progress>
            </div>
        );
    }

});
module.exports = Player;
